<?php
/**
 * Mantis Plugin: Hyperlinks references to Mantis Issues
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Victor Boctor (http://www.futureware.biz)
 */


if ( !defined( 'DOKU_INC' ) ) {
    define( 'DOKU_INC', realpath( dirname( __FILE__ ) . '/../../' ) . '/' );
}
if ( !defined( 'DOKU_PLUGIN' ) ) {
    define( 'DOKU_PLUGIN', DOKU_INC . 'lib/plugins/' );
}
require_once( DOKU_PLUGIN . 'syntax.php' );

/**
 * A plug-in that hyper links references to Mantis issues.  References
 * to Mantis issues should use the following format: ~~Mantis:123~~.
 *
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_mantis extends DokuWiki_Syntax_Plugin {
    
    /**
     * return some info
     */
    function getInfo(){
        return array(
                     'author' => 'Vincent Debout',
                     'email'  => 'deboutv at free . fr',
                     'date'   => '2007-02-18',
                     'name'   => 'Mantis Issues Plugin',
                     'desc'   => 'Support References to Mantis Issues',
                     'url'    => 'http://deboutv.free.fr/mantis/',
                     );
    }
    
    /**
     * What kind of syntax are we?
     */
    function getType(){
        return 'substition'; 
    }
    
    /**
     * What about paragraphs?
     */
    function getPType(){
        return 'normal';
    }
    
    /**
     * Where to sort in?
     */ 
    function getSort(){
        return 156;
    }
    
    
    /**
     * Connect pattern to lexer
     */
    function connectTo( $mode ) {
        $this->Lexer->addSpecialPattern( '~~Mantis:[0-9]+~~', $mode, 'plugin_mantis' );
        $this->Lexer->addSpecialPattern( '~~Mantis:[0-9]+:[a-z_]+~~', $mode, 'plugin_mantis' );
    }
    
    
    /**
     * Handle the match
     */
    function handle( $match, $state, $pos, &$handler ) {
        /* strip "~~Mantis:" from start and "~~" from end */
        $match = str_replace( '~', '', $match );
        $match = explode( ':', $match );
        if ( count( $match ) == 3 ) {
            $t_result[0] = strtolower( $match[1] );
            $t_result[1] = strtolower( $match[2] );
        } else {
            $t_result = strtolower( $match[1] );
        }
        return array( $t_result );
    }
    
    /**
     * Create output
     */
    function render( $format, &$renderer, $data ) {
        if ( $format == 'xhtml' ) {
            if ( is_array( $data[0] ) ) {
                switch( $data[0][1] ) {
                  case 'description':
                  case 'steps_to_reproduce':
                  case 'additional_information':
                    $renderer->doc .= '<pre class="code">' . bug_get_text_field( $data[0][0], $data[0][1] ) . '</pre>';
                    break;
                  case 'summary':
                  case 'category':
                  case 'os':
                  case 'os_build':
                  case 'platform':
                  case 'version':
                  case 'fixed_in_version':
                  case 'target_version':
                  case 'build':
                    $renderer->doc .= bug_get_field( $data[0][0], $data[0][1] );
                    break;
                  case 'date_submitted':
                  case 'last_updated':
                    $renderer->doc .= date( config_get( 'normal_date_format' ), bug_get_field( $data[0][0], $data[0][1] ) );
                    break;
                  default:
                    $renderer->doc .= '~~Mantis:' . $data[0][0] . ':' . $data[0][1] . '~~';
                    break;
                }
            } else {
                $renderer->externallink( config_get( 'path' ) . 'view.php?id=' . $data[0], $data[0] );
            }
            return true;
        }
        return false;
    }
}

?>